package cn.com.qmth.stmms.ms.admin.service;

import cn.com.qmth.stmms.ms.core.domain.InspectRange;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.repository.InspectRangeRepo;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Created by yuanpan on 2017/10/19.
 */
@Service
public class InspectRangeService {

    @Autowired
    private InspectRangeRepo inspectRangeRepo;

    @Autowired
    private PaperRepo paperRepo;


    @Transactional
    public void save(InspectRange inspectRange) {

        InspectRange.InspectRangeType type = inspectRange.getType();

        Long workId = inspectRange.getWorkId();

        List<Paper> papers = paperRepo.findByWorkId(workId);


        List<String> examNumbers = new ArrayList<>();

        Map<String, List<Paper>> group;
        //根据抽查方式，计算出将要抽查的人数
        switch (type) {
            case ZF:
                //根据总分查询
                group = papers.stream().collect(Collectors.groupingBy(Paper::getExamNumber));
                group.forEach((examNumber, paperList) -> {
                    int totalScore = paperList.stream().map(p -> p.getScore().intValue()).reduce((acc, item) -> acc + item).get();
                    if (totalScore > inspectRange.getCondition()) {
                        examNumbers.add(examNumber);
                    }
                });

                break;
            case DKFS:
                //根据单科分数查询
                group = papers.stream().collect(Collectors.groupingBy(Paper::getExamNumber));
                group.forEach((examNumber, paperList) -> {
                    int maxScore = paperList.stream().map(p -> p.getScore().intValue()).max(Integer::compareTo).get();
                    if (maxScore > inspectRange.getCondition()) {
                        examNumbers.add(examNumber);
                    }
                });

                break;
            case SYD:
                //根据生源地排名
                group = papers.stream().collect(Collectors.groupingBy(Paper::getSourceName));

                group.forEach((sourceName, paperList) -> {
                    Map<String, List<Paper>> examNumberList = paperList.stream().collect(Collectors.groupingBy(Paper::getExamNumber));

                    final List<Object[]> totalScoreList = new ArrayList<>();
                    examNumberList.forEach((examNumber, studentPaperList) -> {
                        int totalScore = studentPaperList.stream().map(p -> p.getScore().intValue()).reduce((acc, item) -> acc + item).get();
                        totalScoreList.add(new Object[]{examNumber, totalScore});
                    });

                    totalScoreList.sort((o1, o2) -> {
                        Integer t1 = (Integer) o1[1];
                        Integer t2 = (Integer) o2[1];
                        return t2.compareTo(t1);//分数从高到低排序
                    });

                    int studentCount = totalScoreList.size() > inspectRange.getCondition() ? inspectRange.getCondition() : totalScoreList.size();
                    for (int i = 0; i < studentCount; i++) {
                        Object examNumberInObject = totalScoreList.get(i)[0];
                        examNumbers.add((String) examNumberInObject);
                    }

                });
                break;
        }

        if (inspectRange.getId() != null) {
            //如果是修改操作，要将先前标记的学生试卷改回来
            paperRepo.resetInspectRange(inspectRange.getId());
        }

        //再重新标记
        paperRepo.markInspectRange(inspectRange.getId(), examNumbers);

        inspectRange.setStudentCount(examNumbers.size());
        inspectRangeRepo.save(inspectRange);

    }
}
