package cn.com.qmth.stmms.ms.core.repository;

import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.vo.Subject;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

import java.util.List;
import java.util.stream.Stream;

/**
 * Created by zhengmin on 2016/9/23.
 */
public interface PaperRepo extends JpaRepository<Paper, Long>, JpaSpecificationExecutor<Paper> {

    List<Paper> findByWorkIdAndSubject(Long workId, Subject subject);

    List<Paper> findByWorkIdAndSubject(Long workId, Subject subject, Sort sort);

    List<Paper> findByQuestionId(Long questionId, Sort sort);

    Page<Paper> findByQuestionId(Long questionId, Pageable pageable);

    /**
     * 统计各省各科试卷数量[areaCode,areaName,questionName,count]
     *
     * @param workId
     * @return
     */
    @Query(value = "SELECT q.`area_code`,q.`area_name`,p.`question_name`,COUNT(*) FROM paper p LEFT OUTER JOIN exam_question q ON p.`question_id` = q.`id` WHERE p.`work_id` = ? GROUP BY p.`question_id`,p.`question_name` ORDER BY p.`question_id` ", nativeQuery = true)
    List<Object[]> countGroupByQuestion(Long workId);

    /**
     * 统计科目各试题已评数及总数（分档阶段）
     *
     * @param workId
     * @param subject
     * @return
     */
    @Query(value = "SELECT p.`question_id`,p.question_name,SUM(IF(p.level IS NULL,1,0)),COUNT(p.`question_id`) " +
            "FROM paper p WHERE p.`work_id` = ? AND p.`subject` = ?" +
            "GROUP BY p.`question_id` ,p.question_name ORDER BY p.`question_id`", nativeQuery = true)
    List<Object[]> countGroupByQuestionForLevel(Long workId, String subject);

    /**
     * 统计科目指定试题的各档位数量
     *
     * @param questionId
     * @return
     */
    @Query(value = "SELECT p.`level`,COUNT(*)," +
            "SUM(IF(p.`is_rejected` = 1, 1, 0))," +
            "SUM(IF(p.`is_arbitrated` = 1, 1, 0))" +
            "FROM paper p " +
            "WHERE p.`question_id` = ? " +
            "GROUP BY p.`level` ORDER BY p.`level`", nativeQuery = true)
    List<Object[]> countGroupByLevel(Long questionId);

    /**
     * 统计科目各试题已评数及总数（打分阶段）
     *
     * @param workId
     * @return
     */
    @Query(value = "SELECT p.`question_id`,p.question_name,SUM(IF(p.score IS NULL,1,0)),COUNT(p.`question_id`) " +
            "FROM paper p WHERE p.`work_id` = ? AND p.`subject` = ?" +
            "GROUP BY p.`question_id` ,p.question_name ORDER BY p.`question_id`", nativeQuery = true)
    List<Object[]> countGroupByQuestionForScore(Long workId, String string);

    /**
     * 统计科目未分档的试卷数
     *
     * @param workId
     * @param subject
     * @return
     */
    Long countByWorkIdAndSubjectAndLevelIsNull(Long workId, Subject subject);

    long countByWorkIdAndScoreIsNull(Long workId);

    Long countByWorkId(Long workId);

    long countByWorkIdAndQuestionId(Long workId, Long questionId);

    /**
     * 统计同一个考生的paper数量
     *
     * @param workId
     * @param examNumber
     * @return
     */
    Long countByWorkIdAndExamNumber(Long workId, String examNumber);

    /**
     * 查询指定试卷
     *
     * @param workId
     * @param subject
     * @param examNumber
     * @return
     */
    Paper findByWorkIdAndSubjectAndExamNumber(Long workId, Subject subject, String examNumber);

    /**
     * 查询同一个考生的paper
     *
     * @param workId
     * @param examNumber
     * @return
     */
    List<Paper> findByWorkIdAndExamNumber(Long workId, String examNumber);

    List<Paper> findByWorkIdAndExamNumberIn(Long workId, List<String> examNumbers);

    @Query(value = "select p from Paper p where p.score >= ?1 or p.score <= ?2")
    List<Paper> findByScoreRange(Double highScore, Double lowScore);

    @Query(value = "SELECT p.`exam_number`,p.`student_name`,p.question_id,GROUP_CONCAT(score  ORDER BY p.`subject`) FROM paper p WHERE p.`work_id` = ? GROUP BY p.exam_number,p.`student_name`", nativeQuery = true)
    List<Object[]> findScores(Long workId);

    List<Paper> findByWorkId(Long workId);

    @Modifying
    @Query(nativeQuery = true, value = "update paper set inspect_range = null where inspect_range = :inspectRange")
    void resetInspectRange(@Param("inspectRange") Long inspectRange);


    @Modifying
    @Query(value = "update Paper p set p.inspectRange = ?1 where p.examNumber in ?2")
    void markInspectRange(Long inspectRange, List<String> examNumbers);
}
