package cn.com.qmth.stmms.ms.marking.service;

import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Paper;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.stmms.ms.core.domain.task.MarkTask;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.MarkerGroup;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskRepo;

/**
 * 评卷任务分配
 * Created by zhengmin on 2016/9/23.
 */
@Service
public class AssignTaskService {

    @Autowired
    private MarkTaskRepo markTaskRepo;


    /**
     * 单评任务模式，每个科目评卷员都需要对该科目每份试卷进行评卷
     * 比如，色彩有1000份试卷，那么每个评卷员都需要评1000份
     * @param papers
     * @param markers
     */
    @Transactional
    public void assignForAll(List<Paper> papers, Iterable<MarkUser> markers, MarkSubject markSubject){
        Iterator<Paper> iterator = papers.iterator();
        while(iterator.hasNext()){
            Paper paper = iterator.next();
            for(MarkUser marker : markers){
                MarkTask markTask = new MarkTask(marker,paper,markSubject.getStage());
                markTaskRepo.save(markTask);
                //taskExecutor.execute(papers,marker,stage);
            }
        }
    }

    /**
     * 分组任务模式，将该科目的评卷员分成N组，将该科目试卷平均分给这些组，每组内的评卷员每人将这些试卷打分
     * 比如：6个评卷员分2组，有1000份试卷，每组评500份，每个评卷员评500份
     * @param papers
     * @param markerGroups
     */
    @Transactional
    public void assignForGrouping(List<Paper> papers, List<MarkerGroup> markerGroups,MarkSubject markSubject){
    	markerGroups = markerGroups.stream().filter(m -> m.getMarkers().size()>0).collect(Collectors.toList());
    	int groupSize = markerGroups.size();
        int idx = 0;
        Iterator<Paper> iterator = papers.iterator();
        while(iterator.hasNext()){
            if(idx >= groupSize){
                idx = 0;
            }
            Paper paper = iterator.next();
            for(MarkUser marker : markerGroups.get(idx).getMarkers()){
                MarkTask markTask = new MarkTask(marker,paper,markSubject.getStage());
                markTaskRepo.save(markTask);
            }
            idx++;
        }
    }

}
