package cn.com.qmth.stmms.ms.marking.service;

import java.util.Comparator;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.stmms.ms.core.domain.Level;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTask;
import cn.com.qmth.stmms.ms.core.repository.LevelRepo;

/**
 * 定档
 * Created by zhengmin on 2016/9/22.
 */
@Service
public class DetermineLevelService {

    public enum DeterType{
        /**
         * 多数
         */
        MAJORITY,
        /**
         * 权重
         */
        WEIGHT
    }

    @Autowired
    private LevelRepo levelRepo;

    public DeterResult determine(MarkTask...tasks){
        //如果相同的数量大于半数，直接定档
        Stream<MarkTask> taskStream = Stream.of(tasks);
        Optional<Map.Entry<String,Long>> optional = taskStream
                .collect(Collectors.groupingBy(i -> i.getResult().toString(), Collectors.counting()))
                .entrySet()
                .stream()
                .max(Comparator.comparing(Map.Entry::getValue));
        if(optional.isPresent()){
            Map.Entry<String,Long> entry = optional.get();
            if(entry.getValue() > tasks.length/2){
                return new DeterResult(entry.getKey(),DeterType.MAJORITY);
            }
        }
        //通过权重计算，平均值四舍五入
        int[] values = new int[tasks.length];
        Long workId = null;
        for (int i = 0; i < tasks.length; i++) {
            workId = tasks[i].getWorkId();
            Level level = levelRepo.findByWorkIdAndCode(workId,String.valueOf(tasks[i].getResult()));
            if(level == null){
                throw new RuntimeException("无此档位");
            }
            values[i] = level.getWeight();
        }
        int avg = (int)Math.round(IntStream.of(values).average().orElse(0));
        Optional<Level> levelOptional = levelRepo.findByWorkId(workId).stream().filter(o -> o.getMaxScore() >= avg && o.getMinScore() <= avg).findFirst();
        Level level = levelOptional.orElseThrow(null);
        return new DeterResult(level.getCode(),DeterType.WEIGHT);
    }
}

class DeterResult{
    private String result;
    private DetermineLevelService.DeterType deterType;

    public DeterResult(String result, DetermineLevelService.DeterType deterType) {
        this.result = result;
        this.deterType = deterType;
    }

    public String getResult() {
        return result;
    }

    public void setResult(String result) {
        this.result = result;
    }

    public DetermineLevelService.DeterType getDeterType() {
        return deterType;
    }

    public void setDeterType(DetermineLevelService.DeterType deterType) {
        this.deterType = deterType;
    }
}